package de.lmu.ifi.pst.x10080133.monitor;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Manages every protected request, needs to be registered in deployment descriptor
 * on url-pattern /protected/*
 * 
 * @author Roman.Schwienbacher
 */
public class AppMonitor implements Filter {

	private final Log log = LogFactory.getLog(AppMonitor.class);

	public static final String USERNAME_KEY = "user.name";
	public static final String ROLES_KEY = "user.roles";
	public static final String PRE_VISITED_KEY = "user.pre_visited";
	public static final String REQ_LOCATION_KEY = "user.req_location";
	public static final String MESSAGE_KEY = "user.message";

	private FilterConfig config;

	/**
	 * called at every request, handles corresponding redirecting by the SecurityProvider
	 */
	public void doFilter(ServletRequest req, ServletResponse resp, FilterChain chain) throws IOException, ServletException {
		
		if (((HttpServletRequest) req).getSession().getAttribute(ROLES_KEY) == null) { // check if roles key available
			
			log.debug("no roles key in session, so redirect to login, and write pre-visited and req-location into session");
			
			// read requested location
			String requestedURL = ((HttpServletRequest) req).getRequestURL().toString();
			boolean requestProtectedResource = requestedURL.contains("/protected/");

			if (requestProtectedResource) {

				String requestedLocation = "/protected/" + requestedURL.substring(requestedURL.lastIndexOf("/") + 1, requestedURL.lastIndexOf(".xhtml"));

				((HttpServletRequest) req).getSession().setAttribute(PRE_VISITED_KEY, SecurityProvider.getHomeLocation());
				((HttpServletRequest) req).getSession().setAttribute(REQ_LOCATION_KEY, requestedLocation);

				((HttpServletResponse) resp).sendRedirect("../" + SecurityProvider.getHomeLocation() + ".xhtml");

			} else {

				String requestedLocation = requestedURL.substring(requestedURL.lastIndexOf("/") + 1, requestedURL.lastIndexOf(".xhtml"));

				((HttpServletRequest) req).getSession().setAttribute(PRE_VISITED_KEY, SecurityProvider.getHomeLocation());
				((HttpServletRequest) req).getSession().setAttribute(REQ_LOCATION_KEY, requestedLocation);

				((HttpServletResponse) resp).sendRedirect(SecurityProvider.getHomeLocation() + ".xhtml");
			}

		} else {

			// read requested location
			String requestedURL = ((HttpServletRequest) req).getRequestURL().toString();

			if (requestedURL.endsWith(".css")) { // let css requests pass
				chain.doFilter(req, resp);
			} else {

				log.debug("roles are given in session, user is logged in, so check if he is permitted to acces the requested location");

				String userRolesVal = ((HttpServletRequest) req).getSession().getAttribute(ROLES_KEY).toString();
				String[] userRoles = userRolesVal.split(";");

				// read last visited location out of session
				String preVisitedLocation = ((HttpServletRequest) req).getSession().getAttribute(PRE_VISITED_KEY) == null ? "" : ((HttpServletRequest) req).getSession().getAttribute(PRE_VISITED_KEY).toString();

				boolean requestProtectedResource = requestedURL.contains("/protected/");
				String requestedLocation = requestedURL.substring(requestedURL.lastIndexOf("/") + 1, requestedURL.lastIndexOf(".xhtml"));

				// ask security provider if user is permitted to acces the requestedLocation
				if (SecurityProvider.isAllowedToAccess(userRoles, preVisitedLocation, requestedLocation)) {
					
					log.debug("request of user with roles " + userRolesVal + " and pre visited location " + preVisitedLocation + " on " + requestedLocation + " is granted");
					
					((HttpServletRequest) req).getSession().setAttribute(PRE_VISITED_KEY, requestedLocation); // if so, write requested location into session
					((HttpServletRequest) req).getSession().setAttribute(MESSAGE_KEY, ""); // remove session message
					chain.doFilter(req, resp); // let him pass
					
				} else {				
					
					log.debug("request of user with roles " + userRolesVal + " and pre visited location " + preVisitedLocation + " on " + requestedLocation + " is NOT granted");
					
					// set client message
					((HttpServletRequest) req).getSession().setAttribute(MESSAGE_KEY, "You are not allowed to access the page " + requestedLocation + " with " + preVisitedLocation + " as previous and your role");
					
					// otherwise redirect to corresponding violation page					
					((HttpServletRequest) req).getSession().setAttribute(PRE_VISITED_KEY, SecurityProvider.getViolationLocation(requestedLocation));
					
					if (requestProtectedResource) {
						((HttpServletResponse) resp).sendRedirect("../" + SecurityProvider.getViolationLocation(requestedLocation) + ".xhtml");
					} else {
						((HttpServletResponse) resp).sendRedirect(SecurityProvider.getViolationLocation(requestedLocation) + ".xhtml");
					}
				}
			}
		}
	}

	public FilterConfig getConfig() {
		return config;
	}

	public void init(FilterConfig config) throws ServletException {
		this.config = config;
	}

	public void destroy() {
		config = null;
	}
}
